read("costs.gp")

ps = [751];
maxdepth=[96];
maxmemory=[58,96];

data_to_string(data) = {
  s = "";
  start = 1;
  while(start < matsize(data)[1] && data[start,1] == 0, start += 1);
  prevn = data[start,1];
  for(i=start, matsize(data)[1],
    if(data[i,1] != prevn, s = concat(s,"\n"));
    prevn = data[i,1];
    for(j=1,#data[i,]-1,s = concat([s, round(data[i,j]), ", "]));
    s = concat([s, round(data[i,#data[i,]]), "\n"]));
  s;
}

line_to_string(line) = {
  s = "";
  for(i=1,#line, s = concat([s, " & ", round(line[i])]));
  s;
}

write_tables(filename) = {
  header = "";
  tables = [];
  gap = #maxdepth + 1;
  for(i=1,#ps,
    p=ps[i];
    header = concat([header,    "         &\\multicolumn{3}{c |}{SIKE-$", p, "$}"]));
  header = concat(header, "\\\\\nAttack   ");
  for(i=1,#ps, header=concat(header,"& $G$ & $D$ & $W$ "));
  header = concat(header, "\\\\\n\\hline\n");
  \\ Make tables for maxdepth
  for(i=1,#maxdepth, md=maxdepth[i];
    table = "";
    grover_line = "Grover ";
    tani_line =   "  Tani ";
    vw_line =     "    VW ";
    for(j=1,#ps,p=ps[j];
      t = data_tani_with_maxdepth(p,isogeny_cost(p),md);
      g = data_grover_with_maxdepth(p,isogeny_cost(p),md);
      mm = min(g[4], t[4]);
      v = data_vw_with_maxdepth(p,isogeny_cost(p),md);
      grover_line = concat(grover_line,line_to_string([g[2],g[3],g[4]]));
      tani_line = concat(tani_line,line_to_string([t[2],t[3],t[4]]));
      vw_line = concat(vw_line,line_to_string([v[2],v[3],v[4]])));
    table = concat([header, grover_line, " \\\\\\cline{2-7}\n",  tani_line, " \\\\\\cline{2-7}\n", vw_line, " \\\\\\hline\n"]);
    tables = concat([tables, table]));
  \\ Make tables for maxmemory
  for(i=1,#maxmemory, mm=maxmemory[i];
    table = "";
    grover_line = "Grover ";
    tani_line =   "  Tani ";
    vw_line =     "    VW ";
    for(j=1,#ps,p=ps[j];
      t = data_tani_with_maxmemory(p,isogeny_cost(p),mm);
      g = data_grover_with_maxmemory(p,isogeny_cost(p),mm);
      v = data_vw_with_maxmemory(p,isogeny_cost(p),mm);
      grover_line = concat(grover_line,line_to_string([g[2],g[3],g[4]]));
      tani_line = concat(tani_line,line_to_string([t[2],t[3],t[4]]));
      vw_line = concat(vw_line,line_to_string([v[2],v[3],v[4]])));
    table = concat([header, grover_line, " \\\\\\cline{2-7}\n",  tani_line, " \\\\\\cline{2-7}\n", vw_line, " \\\\\\hline\n"]);
    tables = concat([tables, table]));
  \\ Make G-cost optimal table
  table = "";
  grover_line = "Grover ";
  tani_line =   "  Tani ";
    vw_line =     "    VW ";
  for(j=1,#ps,p=ps[j];
    t = data_tani_min_gates(p,isogeny_cost(p));
    g = data_grover_with_maxdepth(p,isogeny_cost(p),999);
    v = data_vw_with_maxmemory(p,isogeny_cost(p),999);
    grover_line = concat(grover_line,line_to_string([g[2],g[3],g[4]]));
    tani_line = concat(tani_line,line_to_string([t[2],t[3],t[4]]));
    vw_line = concat(vw_line,line_to_string([v[2],v[3],v[4]])));
  table = concat([header, grover_line, " \\\\\\cline{2-7}\n",  tani_line, " \\\\\\cline{2-7}\n", vw_line, " \\\\\\hline\n"]);
  tables = concat([tables, table]);
  \\ Make DW-cost optimal table
  table = "";
  grover_line = "Grover ";
  tani_line =   "  Tani ";
  vw_line =     "    VW ";
  for(j=1,#ps,p=ps[j];
    t = data_tani_min_dw(p,isogeny_cost(p));
    g = data_grover_with_maxdepth(p,isogeny_cost(p),999);
    v = data_vw_with_maxmemory(p,isogeny_cost(p),999);
    grover_line = concat(grover_line,line_to_string([g[2],g[3],g[4]]));
    tani_line = concat(tani_line,line_to_string([t[2],t[3],t[4]]));
  vw_line = concat(vw_line,line_to_string([v[2],v[3],v[4]])));
  table = concat([header, grover_line, " \\\\\\cline{2-7}\n",  tani_line, " \\\\\\cline{2-7}\n", vw_line, " \\\\\\hline\n"]);
  tables = concat([tables, table]);

  write(concat([output_dir,filename,".dat"]), tables);
}

write_tables("table");
quit();
